@@ -184,10 +184,16 @@ void ArgumentCoder<Namespace::Subnamespace::StructName>::encode(OtherEncoder& en
 std::optional<Namespace::Subnamespace::StructName> ArgumentCoder<Namespace::Subnamespace::StructName>::decode(Decoder& decoder)
 {
     auto firstMemberName = decoder.decode<FirstMemberType>();
+    if (UNLIKELY(!firstMemberName))
+        decoder.setIndexOfDecodingFailure(0);
 #if ENABLE(SECOND_MEMBER)
     auto secondMemberName = decoder.decode<SecondMemberType>();
+    if (UNLIKELY(!secondMemberName))
+        decoder.setIndexOfDecodingFailure(1);
 #endif
     auto nullableTestMember = decoder.decode<RetainPtr<CFTypeRef>>();
+    if (UNLIKELY(!nullableTestMember))
+        decoder.setIndexOfDecodingFailure(2);
     if (UNLIKELY(!decoder.isValid()))
         return std::nullopt;
     return {
@@ -1250,6 +1256,43 @@ std::optional<RetainPtr<CFBarRef>> ArgumentCoder<RetainPtr<CFBarRef>>::decode(De
 
 #endif
 
+#if USE(SKIA)
+void ArgumentCoder<SkFooBar>::encode(Encoder& encoder, const SkFooBar& passedInstance)
+{
+    auto instance = CoreIPCSkFooBar(passedInstance);
+    static_assert(std::is_same_v<std::remove_cvref_t<decltype(instance.foo())>, int>);
+    static_assert(std::is_same_v<std::remove_cvref_t<decltype(instance.bar())>, double>);
+
+    encoder << instance.foo();
+    encoder << instance.bar();
+}
+
+void ArgumentCoder<SkFooBar>::encode(OtherEncoder& encoder, const SkFooBar& passedInstance)
+{
+    auto instance = CoreIPCSkFooBar(passedInstance);
+    static_assert(std::is_same_v<std::remove_cvref_t<decltype(instance.foo())>, int>);
+    static_assert(std::is_same_v<std::remove_cvref_t<decltype(instance.bar())>, double>);
+
+    encoder << instance.foo();
+    encoder << instance.bar();
+}
+
+std::optional<SkFooBar> ArgumentCoder<SkFooBar>::decode(Decoder& decoder)
+{
+    auto foo = decoder.decode<int>();
+    auto bar = decoder.decode<double>();
+    if (UNLIKELY(!decoder.isValid()))
+        return std::nullopt;
+    return {
+        CoreIPCSkFooBar {
+            WTFMove(*foo),
+            WTFMove(*bar)
+        }
+    };
+}
+
+#endif
+
 void ArgumentCoder<WebKit::RValueWithFunctionCalls>::encode(Encoder& encoder, WebKit::RValueWithFunctionCalls&& instance)
 {
     static_assert(std::is_same_v<std::remove_cvref_t<decltype(instance.callFunction())>, SandboxExtensionHandle>);
@@ -1450,11 +1493,42 @@ std::optional<Ref<WebCore::AppKitControlSystemImage>> ArgumentCoder<WebCore::App
 
 #endif
 
+void ArgumentCoder<WebCore::RectEdges<bool>>::encode(Encoder& encoder, const WebCore::RectEdges<bool>& instance)
+{
+    static_assert(std::is_same_v<std::remove_cvref_t<decltype(instance.top())>, bool>);
+    static_assert(std::is_same_v<std::remove_cvref_t<decltype(instance.right())>, bool>);
+    static_assert(std::is_same_v<std::remove_cvref_t<decltype(instance.bottom())>, bool>);
+    static_assert(std::is_same_v<std::remove_cvref_t<decltype(instance.left())>, bool>);
+
+    encoder << instance.top();
+    encoder << instance.right();
+    encoder << instance.bottom();
+    encoder << instance.left();
+}
+
+std::optional<WebCore::RectEdges<bool>> ArgumentCoder<WebCore::RectEdges<bool>>::decode(Decoder& decoder)
+{
+    auto top = decoder.decode<bool>();
+    auto right = decoder.decode<bool>();
+    auto bottom = decoder.decode<bool>();
+    auto left = decoder.decode<bool>();
+    if (UNLIKELY(!decoder.isValid()))
+        return std::nullopt;
+    return {
+        WebCore::RectEdges<bool> {
+            WTFMove(*top),
+            WTFMove(*right),
+            WTFMove(*bottom),
+            WTFMove(*left)
+        }
+    };
+}
+
 } // namespace IPC
 
 namespace WTF {
 
-template<> bool isValidEnum<IPC::WebCore_TimingFunction_Subclass, void>(IPC::EncodedVariantIndex value)
+template<> bool isValidEnum<IPC::WebCore_TimingFunction_Subclass>(IPC::EncodedVariantIndex value)
 {
 IGNORE_WARNINGS_BEGIN("switch-unreachable")
     switch (static_cast<IPC::WebCore_TimingFunction_Subclass>(value)) {
@@ -1470,7 +1544,7 @@ IGNORE_WARNINGS_END
     return false;
 }
 
-template<> bool isValidEnum<IPC::WebCore_MoveOnlyBaseClass_Subclass, void>(IPC::EncodedVariantIndex value)
+template<> bool isValidEnum<IPC::WebCore_MoveOnlyBaseClass_Subclass>(IPC::EncodedVariantIndex value)
 {
 IGNORE_WARNINGS_BEGIN("switch-unreachable")
     switch (static_cast<IPC::WebCore_MoveOnlyBaseClass_Subclass>(value)) {
@@ -1481,7 +1555,20 @@ IGNORE_WARNINGS_END
     return false;
 }
 
-template<> bool isValidEnum<EnumWithoutNamespace, void>(uint8_t value)
+#if ENABLE(BOOL_ENUM)
+template<> bool isValidEnum<EnumNamespace::BoolEnumType>(bool value)
+{
+    switch (static_cast<uint8_t>(value)) {
+    case 0:
+    case 1:
+        return true;
+    default:
+        return false;
+    }
+}
+#endif
+
+template<> bool isValidEnum<EnumWithoutNamespace>(uint8_t value)
 {
     switch (static_cast<EnumWithoutNamespace>(value)) {
     case EnumWithoutNamespace::Value1:
@@ -1494,7 +1581,7 @@ template<> bool isValidEnum<EnumWithoutNamespace, void>(uint8_t value)
 }
 
 #if ENABLE(UINT16_ENUM)
-template<> bool isValidEnum<EnumNamespace::EnumType, void>(uint16_t value)
+template<> bool isValidEnum<EnumNamespace::EnumType>(uint16_t value)
 {
     switch (static_cast<EnumNamespace::EnumType>(value)) {
     case EnumNamespace::EnumType::FirstValue:
@@ -1564,7 +1651,7 @@ template<> bool isValidOptionSet<OptionSetEnumAllCondition>(OptionSet<OptionSetE
 }
 
 #if (ENABLE(OUTER_CONDITION)) && (ENABLE(INNER_CONDITION))
-template<> bool isValidEnum<EnumNamespace::InnerEnumType, void>(uint8_t value)
+template<> bool isValidEnum<EnumNamespace::InnerEnumType>(uint8_t value)
 {
     switch (static_cast<EnumNamespace::InnerEnumType>(value)) {
     case EnumNamespace::InnerEnumType::InnerValue:
@@ -1581,6 +1668,19 @@ template<> bool isValidEnum<EnumNamespace::InnerEnumType, void>(uint8_t value)
 }
 #endif
 
+#if (ENABLE(OUTER_CONDITION)) && (!(ENABLE(INNER_CONDITION)))
+template<> bool isValidEnum<EnumNamespace::InnerBoolType>(bool value)
+{
+    switch (static_cast<uint8_t>(value)) {
+    case 0:
+    case 1:
+        return true;
+    default:
+        return false;
+    }
+}
+#endif
+
 } // namespace WTF
 
 IGNORE_WARNINGS_END
